" 读取词典参数 ------------------------------------------------------------{{{1
let s:pageNumDigit = '%0'. dictionaryPart[4] . 'd'
let s:sourceStyle = anyMoreSource[1]
" sourceStyle：定义兼容的源文件或词条格式（MdxSourceBuilderLink.Vim）
" - 0，'一行页码 + 多行关键词（每行一个关键词）'的标准词条格式，跳转至页码
" - 1，'一个页码 + 多个中文单字符的关键词'的压缩词条格式，跳转至页码
" - 2，'页码 + 分隔符 + 单个中或英关键词'的啰嗦行格式，跳转至页码
" - 3，显性化'关键词 + 分隔符 + 现有关键词'的词条格式，跳转至现有关键词
" - 4，隐性化'关键词 + 现有关键词'的词条格式，跳转至现有关键词
" - 999，源文件为标准的mdx源格式，无需进一步处理，直接使用

" mdx源文件格式初始化 -----------------------------------------------------{{{1
" 清理并保存，以便后续代码可以正常运作
silent! normal! Go
silent! global/^$/d
silent! w!

" 将源格式转换为标准的mdx源文件格式 ---------------------------------------{{{1
if s:sourceStyle == 0 || s:sourceStyle == 1 || s:sourceStyle == 2
    " 将input词条格式标准化 -----------------------------------------------{{{2
    if s:sourceStyle == 0
        " 适合如下标准词条格式：一行页码 + 多行关键词（每行一个关键词）
        " 0001
        " a
        " b
        " c
        " 0002
        " x
        " y
        " z
    elseif s:sourceStyle == 1
        " 适合如下词条格式：行格式为'一个页码+多个中文单字符的关键词'
        " 0001吖阿啊锕腌啊嗄啊哎
        " 0002哀埃挨唉锿挨皑癌毐欸嗳矮蔼
        " 0003霭艾砹唉爱隘碍嗳嗌媛瑷

        " 以下将的单行格式转换为标准词条格式
        " 将页码换行
        silent! %s/^\d\{3,}\($\)\@!/\0\r/e
        " 将每个字独立成行
        silent! %s/\D\($\)\@!/\0\r/ge
    elseif s:sourceStyle == 2
        " 适合如下词条格式：行格式为"页码 + 分隔符 + 单个中或英关键词"
        " 分隔符兼容：Tab键'\t', 4个及以上空格'\s\{4,}'
        " 0001    abandon
        " 0001    abandoned
        " 0002    abandonee
        " 0002    a bas
        " 0003    abdominous

        normal gg
        " 将所有Tab替换为空格
        silent! set expandtab tabstop=4 | %retab
        let linenumber = printf(s:pageNumDigit, 0)
        for line in getline(1,'$')
            let words = split(line, '\s\{4,}')
            if linenumber == words[0]
                silent! s/^.*$/\= words[1]/e
            else
                let linenumber = words[0]
                silent! s/^.*$/\= words[0] . "\n". get(words, 1, "")/e
            endif
            silent! normal j
        endfor
    endif

    " 清理并保存，以便后续代码可以正常运作
    silent! normal! Go
    silent! global/^$/d
    silent! w!

    " 创建pageList 和 keywordsDict ----------------------------------------{{{2

    " 创建、排序、去重 pageList
    let s:pageList = []
    global/^\d\{3,}$/call add(s:pageList, str2nr(getline('.')))
    let s:pageList = uniq(sort(s:pageList,'n'))
    " echomsg s:pageList

    " 创建 keywordsDict
    let s:keywordsDict = {}
    silent! vimgrep /^\d\{3,}$/ %
    function! KeywordsDict()
        let startline = line(".")
        let currentPage = str2nr(getline('.'))

        silent! cbelow
        let endline = line(".")
        let lastline = line("$")

        if endline == startline
            if endline == lastline
                silent! let s:keywordsDict[currentPage]
                            \= get(s:keywordsDict, currentPage, [])
                            \+ getline(startline+1, endline-1)
            else
                silent! let s:keywordsDict[currentPage]
                            \= get(s:keywordsDict, currentPage, [])
                            \+ getline(startline+1, lastline)
            endif
        else
            silent! let s:keywordsDict[currentPage]
                        \= get(s:keywordsDict, currentPage, [])
                        \+ getline(startline+1, endline-1)
            silent! cabove
        endif
        return s:keywordsDict
    endfunction
    silent! cdo call KeywordsDict()
    " echomsg s:keywordsDict

    " 清空全文
    silent! normal! ggdG

    " 将标准化input词条转为页面导航源文件格式 -----------------------------{{{2
    " 自身没有页面和keywords导航，仅转LINK
    for currentPage in s:pageList
        for currentKeyword in s:keywordsDict[currentPage]
            " 对currentKeyword进行特殊处理; 若不需要则注释掉
            let currentKeyword = "s" . split(currentKeyword, '\s')[0]
            silent! let @k = currentKeyword . "\n"
                \. '@@@LINK=' . anyMoreSource[2]
                \. printf(s:pageNumDigit, currentPage)
                \. "\n". '</>'
            silent! $put k
        endfor
    endfor
elseif s:sourceStyle == 3
    " todo
elseif s:sourceStyle == 4
    " 将带有编号的主词条改造为使用"s+编号"进行检索
    silent! %s/^\(\d*\)\(\s.*\)$/s\1\r@@@LINK=\0\r<\/>/
elseif s:sourceStyle == 999
    " todo
else
    echomsg "请定义 SourceStyle " . s:sourceStyle . " 的处理方案"
endif

" 将输出结果保存到mdxSource  ----------------------------------------------{{{1
silent! global/^$/d
silent! normal! gg"xyG
let g:mdxSource = g:mdxSource . @x

" Reference  --------------------------------------------------------------{{{1
finish

" 在新Tab页打开output文件
silent! tabe %:r.output.%:e

/* vim: set et sw=4 ts=4 sts=4 fdm=marker ff=unix ft=vim fenc=utf8 nobomb: */
